#include "testApp.h"

using namespace ofxCv;
using namespace cv;

const float dyingTime = 1;

void Glow::setup(const cv::Rect& track) {
	color.setHsb(ofRandom(0, 255), 255, 255);
	cur = toOf(track).getCenter();
	smooth = cur;
	numVert = 100;
}

void Glow::update(const cv::Rect& track) {
	cur = toOf(track).getCenter();
	smooth.interpolate(cur, .5);
	all.addVertex(smooth);
	if(all.size() > numVert) {
	    vector<ofPoint> pts;
		pts = all.getVertices();
		pts.erase(pts.begin());
		all.clear();
		all.addVertices(pts);
	}
}

void Glow::kill() {
	float curTime = ofGetElapsedTimef();
	if(startedDying == 0) {
		startedDying = curTime;
//	} else if(curTime - startedDying > dyingTime) {
//		dead = true;
	} else if(all.size() > 0) {
	    vector<ofPoint> pts;
		pts = all.getVertices();
		pts.erase(pts.begin());
		all.clear();
		all.addVertices(pts);
	} else {
	    dead = true;
	}
}

void Glow::draw() {
	ofPushStyle();
	float size = 16;
//	ofSetColor(255);
//	if(startedDying) {
//		ofSetColor(ofColor::red);
//		size = ofMap(ofGetElapsedTimef() - startedDying, 0, dyingTime, size, 0, true);
//	}
//	ofNoFill();
//	ofCircle(cur, size);
	ofSetColor(color);
	ofSetLineWidth(4);
	all.draw();
//	ofSetColor(255);
//	ofDrawBitmapString(ofToString(label), cur);
	ofPopStyle();
}

void testApp::setup() {
	ofSetVerticalSync(true);
	ofBackground(0);
	//ofEnableSmoothing();

    camWidth = 640;
    camHeight = 480;
    //Glow::numVert = 100;

	camera.setVerbose(true);
	camera.setDesiredFrameRate(30);
	camera.initGrabber(camWidth, camHeight);

	camImg.allocate(camWidth, camHeight, OF_IMAGE_COLOR);

	contourFinder.setMinAreaRadius(1);
	contourFinder.setMaxAreaRadius(50);
	contourFinder.setThreshold(190);

	// wait for half a frame before forgetting something
	tracker.setPersistence(15);
	// an object can move up to 32 pixels per frame
	tracker.setMaximumDistance(32);
}

void testApp::update() {
    ofWidth = ofGetWidth();
    ofHeight = ofGetHeight();

	camera.update();
	if(camera.isFrameNew()) {
	    camImg.setFromPixels(camera.getPixelsRef());
        camImg.mirror(false, true);
        blur(camImg, 10);
		contourFinder.findContours(camImg);
		tracker.track(contourFinder.getBoundingRects());

//		blur(camera, 10);
//		contourFinder.findContours(camera);
//		tracker.track(contourFinder.getBoundingRects());
	}
}

void testApp::draw() {
	ofSetColor(255);
//  camera.draw(0, 0);
    camImg.draw(0, 0, ofWidth, ofHeight);
//	contourFinder.draw();

    ofPushMatrix();
    ofScale(ofWidth/(float(camWidth)), ofHeight/(float(camHeight)), 0.0);
	vector<Glow>& followers = tracker.getFollowers();
	for(int i = 0; i < followers.size(); i++) {
		followers[i].draw();
	}
	ofPopMatrix();
}

void testApp::keyPressed(int key) {
	if (key == 'f') {
		ofToggleFullscreen();
	}
}
